/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2012 digitalSTROM.org, Zurich, Switzerland
 *  Author: Roman Khler
 */



function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}

Ext.define('DSS.addon.HLEEdit', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : 'User defined action',
        appId : 'hleedit',
        appVersion : '1.0.0.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            loader : {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },

    getContent : function() {
        var me=this;
        var oConfig={
            defaultSortCol:'name',
            disabledProperty:'disabled',
            storeConfig:{
                storeId : 'hleStore',
                fields : [ {name:'id'}, 
                           {name:'disabled', sortType:'asBoolean'}, 
                           {name:'name', sortType:'asUCText'}, 
                           {name:'actions', sortType:'asAction'}, 
                           {name:'lastSaved', sortType:'asTimestamp'}, 
                           {name:'lastExecuted', sortType:'asTimestamp'} ]
            },
            multiselect:true,
            columns:[
                {
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                }, 
                {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width : 200
                },
                {
                    header : _('Activity'),
                    dataIndex : 'actions', 
                    maintainFlex : true,
                    flex : 5,
                    xtype: 'actioncolumn'
                },
                {
                    header : _('last edit'),
                    dataIndex : 'lastSaved',
                    xtype: 'timestampcolumn',
                    width : 200
                }/*,
                {
                    header : _('last executed'),
                    dataIndex : 'lastExecuted',
                    xtype:'timestampcolumn',
                    width : 200
                }*/
            ],
            buttonText: {
                newButton: _('New action'),
                editButton: _('Edit action'),
                enableButton:_('Activate action'),
                disableButton:_('Deactivate action'),
                testButton: _('Test action'),
                deleteButton: _('Delete action')
            },
            editorConfig:{
                title : _('Edit user defined action'),
                staticControls : [ {
                    type : 'text',
                    label : _('Name'),
                    id : 'name'
                }, {
                    type : 'hidden',
                    label : 'id',
                    id : 'id'
                } ],
                tabControls : [ {
                    tabLabel : _('Activity'),
                    tabDescription : _('Choose the activities, which should be executed.'),
                    content : [ {
                        type : 'ActionConfigurator',
                        id : 'actions',
                        flex : true,
                        noHighlevel:true
                    } ]
                }, {
                    tabLabel : _('Conditions'),
                    content : []
                } ]
            },
            listeners:{
                'loadDataStore':function (grid){
                    me.loadParameter(grid);
                },
                'saveEntry':function (grid,values){                        
                    dss.raiseEvent('system-addon-user-defined-actions-config', {
                        actions : 'save',
                        value : Ext.JSON.encode(values)
                    })
                    me.waitAndReload(grid);
                },
                'enableEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    dss.raiseEvent('system-addon-user-defined-actions-config', {
                        actions : 'enable',
                        id : Ext.JSON.encode(idVal)
                    });
                    me.waitAndReload(grid);
                 },
                'disableEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    dss.raiseEvent('system-addon-user-defined-actions-config', {
                        actions : 'disable',
                        id : Ext.JSON.encode(idVal)
                    })
                    me.waitAndReload(grid);
                },
                'deleteEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    dss.raiseEvent('system-addon-user-defined-actions-config', {
                        actions : 'delete',
                        id : Ext.JSON.encode(idVal)
                    })
                    me.waitAndReload(grid);
                },
                'testEntry':function (grid,entry){
                    dss.raiseEvent('highlevelevent', {
                        id : entry.get('id')
                    });
                    window.setTimeout(function() {
                        me.loadParameter(grid);
                    }, 2 * 1000);
                }

            }
        };
        
        if (getParameterByName('mode')=='expert') {
            oConfig.editorConfig.tabControls[0].content[0].withDelay=true;
            oConfig.editorConfig.tabControls[0].content[0].allowNoAction=true;
            oConfig.editorConfig.tabControls[0].content[0].noHighlevel=false;
            oConfig.editorConfig.tabControls[0].content[0].enableUsergroup=true;
        }
        return Ext.create('DSS.gridPanel', oConfig);
    },
    waitAndReload : function(grid) {
    	grid.setStatus(_('Saving ...'));
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'system-addon-user-defined-actions-saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter(grid);
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'system-addon-user-defined-actions-saved'
            });
        });
    },

    loadParameter : function(grid) {
    	grid.setStatus(_('Loading ...'));
        dss.staticDataModel.initHLECache();
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/usr/events/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        var oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                               {name:'id',type:'simple'},
                                                               {name:'disabled',type:'condition/enabled'},
                                                               {name:'lastSaved',type:'simple'},
                                                               {name:'lastExecuted',type:'simple'},
                                                               {name:'actions',type:'action'}]);
        if (oResultObject!=null) {
            grid.loadParameter(oResultObject);
        }
        grid.setStatus(_('Ready'));
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/user-defined-actions.po']);
    var HLEEdit = Ext.create('DSS.addon.HLEEdit');
    HLEEdit.initPage();
});
